
#ifndef __PathPlanning_WINDOW_H_
#define __PathPlanning_WINDOW_H_

#include <VirtualRobot/VirtualRobot.h>
#include <VirtualRobot/Robot.h>
#include <VirtualRobot/VirtualRobotException.h>
#include <VirtualRobot/Nodes/RobotNode.h>
#include <VirtualRobot/XML/RobotIO.h>
#include <VirtualRobot/Visualization/VisualizationFactory.h>
#include <VirtualRobot/Obstacle.h>
#include <VirtualRobot/RobotNodeSet.h>
#include <VirtualRobot/Visualization/CoinVisualization/CoinVisualizationNode.h>
#include <VirtualRobot/Visualization/CoinVisualization/CoinVisualization.h>

#include <string.h>
#include <QtCore/QtGlobal>
#include <QtGui/QtGui>
#include <QtCore/QtCore>

#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/Qt/SoQt.h>
#include <Inventor/nodes/SoSeparator.h>

#include "Planner/Planner2D.h"

#include <vector>

#include "ui_PathPlanning.h"

class PathPlanningWindow : public QMainWindow
{
	Q_OBJECT
public:
    PathPlanningWindow(Qt::WFlags flags = 0);
    ~PathPlanningWindow();

	/*!< Executes the SoQt mainLoop. You need to call this in order to execute the application. */
	int main();

    bool loadRobot(std::string robotFilename);

public slots:
	/*! Closes the window and exits SoQt runloop. */
	void quit();

	/*!< Overriding the close event, so we know when the window was closed by the user. */
	void closeEvent(QCloseEvent *event);

	void resetSceneryAll();

    void loadScene(std::string sceneFile);

    void checkBoxOption1();

    void buttonRunPlanner();
    void buttonStep1();
    void buttonStep2();
    void buttonStep3();

    void sliderTrajectory(int pos);

    void comboBoxSelectScene(int scene);
    void comboBoxSelectAlgorithm(int algo);
    void rebuildVisualization();

protected:
	void setupUI();


    void showTrajectory(std::vector< Eigen::Vector2f > &traj);

    std::string robotFilename;
    Ui::PathPlanning UI;
	SoQtExaminerViewer *viewer; /*!< Viewer to display the 3D model of the robot and the environment. */
		
	SoSeparator *sceneSep;
    SoSeparator *robotSep;
    SoSeparator *obstacleSep;
    SoSeparator *floorSep;
    SoSeparator *trajectorySep;

    Planner2DPtr p;
    VirtualRobot::SceneObjectSetPtr obstacles;
    VirtualRobot::SceneObjectPtr floor;

	VirtualRobot::RobotPtr robot;
    VirtualRobot::ScenePtr scene;
    std::vector< std::string > sceneFiles;

    std::vector< Eigen::Vector2f > trajectory;

    boost::shared_ptr<VirtualRobot::CoinVisualization> visualization;
};

#endif // __PathPlanning_WINDOW_H_
